@@ -5,6 +5,7 @@
  */
 
 #include "kcolorschemewatcher_win.h"
+#include "kcolorschemewatcher.h"
 
 #include <windows.h>
 
@@ -12,11 +13,21 @@
 
 #include <QDebug>
 
+static bool isHighContrastModeActive()
+{
+    HIGHCONTRAST result;
+    result.cbSize = sizeof(HIGHCONTRAST);
+    if (SystemParametersInfo(SPI_GETHIGHCONTRAST, result.cbSize, &result, 0)) {
+        return (result.dwFlags & HCF_HIGHCONTRASTON);
+    }
+    return false;
+}
+
 KColorSchemeWatcherWin::KColorSchemeWatcherWin()
 {
     QAbstractEventDispatcher::instance()->installNativeEventFilter(this);
 
-    m_preferDarkMode = !(m_settings.value(QStringLiteral("AppsUseLightTheme"), true).value<bool>());
+    m_preference = getCurrentSystemPreference();
 }
 
 bool KColorSchemeWatcherWin::nativeEventFilter(const QByteArray &eventType, void *message, qintptr *)
@@ -25,9 +36,9 @@ bool KColorSchemeWatcherWin::nativeEventFilter(const QByteArray &eventType, void
     switch (msg->message) {
     case WM_SETTINGCHANGE: {
         m_settings.sync();
-        const bool preferDarkModeNow = !(m_settings.value(QStringLiteral("AppsUseLightTheme"), true).value<bool>());
-        if (m_preferDarkMode != preferDarkModeNow) {
-            m_preferDarkMode = preferDarkModeNow;
+        const auto currentPreference = getCurrentSystemPreference();
+        if (m_preference != currentPreference) {
+            m_preference = currentPreference;
             Q_EMIT systemPreferenceChanged();
         }
 
@@ -41,7 +52,17 @@ bool KColorSchemeWatcherWin::nativeEventFilter(const QByteArray &eventType, void
 
 KColorSchemeWatcher::ColorPreference KColorSchemeWatcherWin::systemPreference() const
 {
-    return m_preferDarkMode ? KColorSchemeWatcher::PreferDark : KColorSchemeWatcher::PreferLight;
+    return m_preference;
+}
+
+KColorSchemeWatcher::ColorPreference KColorSchemeWatcherWin::getCurrentSystemPreference() const
+{
+    if (isHighContrastModeActive()) {
+        return KColorSchemeWatcher::PreferHighContrast;
+    } else if (m_settings.value(QStringLiteral("AppsUseLightTheme"), true).value<bool>()) {
+        return KColorSchemeWatcher::PreferLight;
+    }
+    return KColorSchemeWatcher::PreferDark;
 }
 
 #include "moc_kcolorschemewatcher_win.cpp"
